/*
  Steppers_Aclr8_2.h - Stepper library for two (synchronous stepper motors
  with acceleration and deceleration for Wiring/Arduino 
  - Version 1.0
 */

// ensure this library description is only included once
#ifndef Steppers_Aclr8_2_h
#define Steppers_Aclr8_2_h

// library interface description
class Steppers_Aclr8_2 {
  public:
 
	// Initialise he class
	Steppers_Aclr8_2();

	// Stepper constructors: 4 wire stepper motors only
	void stepper1(int number_of_steps1, int motor1_pin_1, \
		int motor1_pin_2, int motor1_pin_3, int motor1_pin_4);
	void stepper2(int number_of_steps2, int motor2_pin_1, \
		int motor2_pin_2, int motor2_pin_3, int motor2_pin_4);

	// Define each stepper's rates (min speed, max speed, acceleration)
    void stepper1speeds(long minSpeed1, long maxSpeed1, double Accel1);
    void stepper2speeds(long minSpeed2, long maxSpeed2, double Accel2);

    // Create the step method
    void step(int steps_to_move1, int steps_to_move2);

    int version(void);

  private:
	// setup the motors
    void stepMotor1(int thisStep1);
    void stepMotor2(int thisStep2);
	
	// Direction of rotation
	int direction1;        
	int direction2;
    
	// Speed in RPMs
	int speed1;          
	int speed2;          
    
	// delay between steps, in ms, based on speed1
    double step_delay1;    
    double step_delay2;    
	
	// total number of steps this motor can take
	int number_of_steps1;      
	int number_of_steps2;      
    
	// delay at 1 rev per minute
    double delay_1revpermin1;
    double delay_1revpermin2;
	
	// which step the motor is on
	int step_number1;        
	int step_number2;        
	
	//steps left
	int steps_left1;    
	int steps_left2;    

	// motor pin numbers:
    int motor1_pin_1;
    int motor2_pin_1;

    int motor1_pin_2;
    int motor2_pin_2;

    int motor1_pin_3;
	int motor2_pin_3;

	int motor1_pin_4;
    int motor2_pin_4;
    
	// time stamp in ms of when the last step was taken
	long last_step_time1;      
	long last_step_time2;      

	//acceleration
	long motorMinSpeed1;
	long motorMinSpeed2;
	
	long motorMaxSpeed1;
	long motorMaxSpeed2;
	
	long motorAccSteps1;
	long motorAccSteps2;
	
	double motorAccelVal1;
	double motorAccelVal2;

	// which step number of steps to take
	int steps_no1;
	int steps_no2;

	// total steps for both motors
	int total_steps_left;	
};

#endif